@@ -22,7 +22,9 @@ package org.apache.cxf.transport.websocket.undertow;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.PrintWriter;
+import java.net.URLEncoder;
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.util.Collection;
 import java.util.Locale;
 import java.util.Map;
@@ -30,11 +32,10 @@ import java.util.TreeMap;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.servlet.ServletOutputStream;
-import javax.servlet.WriteListener;
-import javax.servlet.http.Cookie;
-import javax.servlet.http.HttpServletResponse;
-
+import jakarta.servlet.ServletOutputStream;
+import jakarta.servlet.WriteListener;
+import jakarta.servlet.http.Cookie;
+import jakarta.servlet.http.HttpServletResponse;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.transport.websocket.WebSocketConstants;
 import org.apache.cxf.transport.websocket.WebSocketUtils;
@@ -189,37 +190,7 @@ public class WebSocketUndertowServletResponse implements HttpServletResponse {
         return responseHeaders.containsKey(name);
     }
 
-    @Override
-    public String encodeRedirectURL(String url) {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.log(Level.FINE, "encodeRedirectURL({0})", url);
-        }
-        return null;
-    }
-
-    @Override
-    public String encodeRedirectUrl(String url) {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.log(Level.FINE, "encodeRedirectUrl({0})", url);
-        }
-        return null;
-    }
-
-    @Override
-    public String encodeURL(String url) {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.log(Level.FINE, "encodeURL({0})", url);
-        }
-        return null;
-    }
-
-    @Override
-    public String encodeUrl(String url) {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.log(Level.FINE, "encodeUrl({0})", url);
-        }
-        return null;
-    }
+    
 
     @Override
     public String getHeader(String name) {
@@ -308,14 +279,6 @@ public class WebSocketUndertowServletResponse implements HttpServletResponse {
         responseHeaders.put(WebSocketUtils.SC_KEY, Integer.toString(sc));
     }
 
-    @Override
-    public void setStatus(int sc, String sm) {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.log(Level.FINE, "setStatus({0}, {1})", new Object[]{sc, sm});
-        }
-        responseHeaders.put(WebSocketUtils.SC_KEY, Integer.toString(sc));
-    }
-
     private ServletOutputStream createOutputStream() {
         //REVISIT
         // This output buffering is needed as the server side websocket does
@@ -389,4 +352,29 @@ public class WebSocketUndertowServletResponse implements HttpServletResponse {
         throw new UnsupportedOperationException();
 
     }
+
+    @Override
+    public String encodeURL(String url) {
+        return URLEncoder.encode(url, StandardCharsets.UTF_8);
+    }
+
+    @Override
+    public String encodeRedirectURL(String url) {
+        return URLEncoder.encode(url, StandardCharsets.UTF_8);
+    }
+
+
+    public void setStatus(int sc, String sm) {
+        if (LOG.isLoggable(Level.FINE)) {
+            LOG.log(Level.FINE, "setStatus({0})", sc);
+        }
+        responseHeaders.put(WebSocketUtils.SC_KEY, Integer.toString(sc));
+    }
+    
+    @Override
+    public void sendRedirect(String location, int sc, boolean clearBuffer) throws IOException {
+        if (LOG.isLoggable(Level.FINE)) {
+            LOG.log(Level.FINE, "sendRedirect({0}, {1}, {2})", new Object[]{location, sc, clearBuffer});
+        }
+    }
 }
