@@ -27,8 +27,6 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
-import com.fasterxml.jackson.jakarta.rs.json.JacksonJsonProvider;
-
 import jakarta.ws.rs.core.GenericType;
 import jakarta.ws.rs.core.MediaType;
 import org.apache.cxf.BusFactory;
@@ -38,7 +36,9 @@ import org.apache.cxf.jaxrs.client.WebClient;
 import org.apache.cxf.jaxrs.ext.multipart.Attachment;
 import org.apache.cxf.jaxrs.model.AbstractResourceInfo;
 import org.apache.cxf.jaxrs.provider.JAXBElementProvider;
+import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
+import tools.jackson.jakarta.rs.json.JacksonJsonProvider;
 
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
@@ -58,11 +58,13 @@ public class JAXRSClientServerResourceJacksonSpringProviderTest extends Abstract
         assertTrue("server did not launch correctly",
                    launchServer(BookServerResourceJacksonSpringProviders.class, true));
         createStaticBus();
-        BusFactory.getDefaultBus().setProperty("skip.default.json.provider.registration", true);
+        BusFactory.getDefaultBus().setProperty(ProviderFactory.SKIP_JAKARTA_JSON_PROVIDERS_REGISTRATION, true);
+        BusFactory.getDefaultBus().setProperty(ProviderFactory.SKIP_DEFAULT_JSON_PROVIDER_REGISTRATION, true);
     }
     @AfterClass
     public static void afterClass() throws Exception {
-        BusFactory.getDefaultBus().getProperties().remove("skip.default.json.provider.registration");
+        BusFactory.getDefaultBus().getProperties().remove(ProviderFactory.SKIP_JAKARTA_JSON_PROVIDERS_REGISTRATION);
+        BusFactory.getDefaultBus().getProperties().remove(ProviderFactory.SKIP_DEFAULT_JSON_PROVIDER_REGISTRATION);
     }
 
     @Test
@@ -77,7 +79,7 @@ public class JAXRSClientServerResourceJacksonSpringProviderTest extends Abstract
         assertNotNull(in);
 
         assertEquals("Jackson output not correct",
-                     "{\"class\":\"org.apache.cxf.systest.jaxrs.Book\",\"name\":\"CXF in Action\",\"id\":123}",
+                     "{\"class\":\"org.apache.cxf.systest.jaxrs.Book\",\"id\":123,\"name\":\"CXF in Action\"}",
                      getStringFromInputStream(in).trim());
     }
 
