@@ -23,9 +23,8 @@ import java.util.Collections;
 import java.util.HashSet;
 import java.util.Set;
 
-import javax.ws.rs.core.Application;
-
-import com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider;
+import jakarta.ws.rs.core.Application;
+import tools.jackson.jakarta.rs.json.JacksonJsonProvider;
 
 public class SseApplication extends Application {
     @Override
