@@ -23,12 +23,10 @@ import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Set;
 
-import javax.ws.rs.ApplicationPath;
-import javax.ws.rs.core.Application;
-
-import com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider;
-
+import jakarta.ws.rs.ApplicationPath;
+import jakarta.ws.rs.core.Application;
 import org.apache.cxf.systest.jaxrs.resources.Library;
+import tools.jackson.jakarta.rs.json.JacksonJsonProvider;
 
 @ApplicationPath("app")
 public class LibraryApplication extends Application {
